package com.css.simulation.resource.scheduler.service;

import com.css.simulation.resource.scheduler.configuration.redis.CustomRedisClient;
import com.css.simulation.resource.scheduler.data.entity.PrefixEntity;
import com.css.simulation.resource.scheduler.data.entity.ProjectEntity;
import com.css.simulation.resource.scheduler.data.entity.TaskEntity;
import com.css.simulation.resource.scheduler.mapper.TaskMapper;
import com.css.simulation.resource.scheduler.util.ProjectUtil;
import com.css.simulation.resource.scheduler.util.TaskUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;

@Service
@Slf4j
public class TaskService {
    @Resource
    private TaskMapper taskMapper;
    @Resource
    private ProjectUtil projectUtil;
    @Resource
    private TaskUtil taskUtil;
    @Resource
    private CustomRedisClient customRedisClient;


    // -------------------------------- Comment --------------------------------

    @Async("pool1")
    public void state(String taskId, String state, String podName) {
        TaskEntity taskEntity = taskMapper.selectById(taskId);
        String projectId = taskEntity.getPId(); // 项目 id
        String lock1 = "taskId:" + taskId + ":state:" + state + ":pod-name:" + podName;

        customRedisClient.lock(lock1, 1L, 30 * 60L);
        try {
            // 查询相关信息
            ProjectEntity projectEntity = projectUtil.getProjectByProjectId(projectId);
            String projectType = projectEntity.getProjectType();  // 项目类型
            String maxSimulationTime = projectEntity.getMaxSimulationTime();  // 项目类型
            String userId = taskEntity.getCreateUserId();   // 用户 id
            PrefixEntity redisPrefix = projectUtil.getRedisPrefixByUserIdAndProjectIdAndTaskId(userId, projectId, taskId);
            // 判断是否完成
            taskUtil.isProjectCompleted(redisPrefix, userId, projectId, projectType, maxSimulationTime, taskId, state, podName);
        } finally {
            customRedisClient.unlock(lock1);
        }

    }



    public Boolean confirm(String taskId) {
        return taskUtil.taskConfirm(taskId);
    }

    public void tick(String taskId) {
        taskUtil.taskTick(taskId);
    }
}



